@@ -3,7 +3,11 @@ from unittest.mock import Mock, call
 import numpy as np
 
 from psygnal import SignalGroup
-from psygnal.containers import EventedObjectProxy, _evented_proxy
+from psygnal.containers import (
+    EventedCallableObjectProxy,
+    EventedObjectProxy,
+    _evented_proxy,
+)
 from psygnal.utils import monitor_events
 
 
@@ -32,7 +36,7 @@ def test_evented_proxy():
         t.x = 2
         t.f = "f"
         del t.x
-        setattr(t, "y", "new")
+        t.y = "new"
         t[0] = 7
         t[0] = 7  # no event
         del t[0]
@@ -154,6 +158,6 @@ def test_evented_callable_proxy():
     def f(*args, **kwargs):
         calls.append((args, kwargs))
 
-    ef = EventedObjectProxy(f)
+    ef = EventedCallableObjectProxy(f)
     ef(1, 2, foo="bar")
     assert calls == [((1, 2), {"foo": "bar"})]
